@@ -27,8 +27,6 @@
                                   see Recursive.''
                                         -- Unknown   */
 #include "ssl_private.h"
-#include <apr_md5.h>
-
 #include "mod_ssl.h"
 #include "mod_ssl_openssl.h"
 #include "mpm_common.h"
@@ -189,110 +187,6 @@ static void ssl_add_version_components(apr_pool_t *ptemp, apr_pool_t *pconf,
                  modver, AP_SERVER_BASEVERSION, incver);
 }
 
-#ifdef HAVE_TLSEXT
-/* Helper functions to create the SNI vhost policy hash. The policy
- * hash captures the configuration elements relevant to the mode
- * selected at runtime by SSLVHostSNIPolicy. */
-
-#define md5_str_update(ctx_, pfx_, str_) do { apr_md5_update(ctx_, pfx_, strlen(pfx_)); apr_md5_update(ctx_, str_, strlen(str_)); } while (0)
-#define md5_ifstr_update(ctx_, pfx_, str_) do { apr_md5_update(ctx_, pfx_, strlen(pfx_)); if (str_) apr_md5_update(ctx_, str_, strlen(str_)); } while (0)
-#define md5_fmt_update(ctx_, fmt_, i_) do { char s_[128]; apr_snprintf(s_, sizeof s_, fmt_, i_); \
-        apr_md5_update(ctx_, s_, strlen(s_)); } while (0)
-
-static int md5_strarray_cmp(const void *p1, const void *p2)
-{
-    return strcmp(*(char **)p1, *(char **)p2);
-}
-
-/* Hashes an array of strings in sorted order. */
-static void md5_strarray_hash(apr_pool_t *ptemp, apr_md5_ctx_t *hash,
-                              const char *pfx, apr_array_header_t *s)
-{
-    char **elts = apr_pmemdup(ptemp, s->elts, s->nelts * sizeof *elts);
-    int i;
-
-    qsort(elts, s->nelts, sizeof(char *), md5_strarray_cmp);
-
-    apr_md5_update(hash, pfx, strlen(pfx));
-    for (i = 0; i < s->nelts; i++) {
-        md5_str_update(hash, "elm:", elts[i]);
-    }
-}
-
-static void hash_sni_policy_pk(apr_pool_t *ptemp, apr_md5_ctx_t *hash, modssl_ctx_t *ctx)
-{
-    md5_fmt_update(hash, "protocol:%d", ctx->protocol);
-
-    md5_ifstr_update(hash, "ciphers:", ctx->auth.cipher_suite);
-    md5_ifstr_update(hash, "tls13_ciphers:", ctx->auth.tls13_ciphers);
-
-    md5_strarray_hash(ptemp, hash, "cert_files:", ctx->pks->cert_files);
-    md5_strarray_hash(ptemp, hash, "key_files:", ctx->pks->key_files);
-}
-
-static void hash_sni_policy_auth(apr_md5_ctx_t *hash, modssl_ctx_t *ctx)
-{
-    modssl_pk_server_t *pks = ctx->pks;
-    modssl_auth_ctx_t *a = &ctx->auth;
-
-    md5_fmt_update(hash, "verify_depth:%d", a->verify_depth);
-    md5_fmt_update(hash, "verify_mode:%d", a->verify_mode);
-
-    md5_ifstr_update(hash, "ca_name_path:", pks->ca_name_path);
-    md5_ifstr_update(hash, "ca_name_file:", pks->ca_name_file);
-    md5_ifstr_update(hash, "ca_cert_path:", a->ca_cert_path);
-    md5_ifstr_update(hash, "ca_cert_file:", a->ca_cert_file);
-    md5_ifstr_update(hash, "crl_path:", ctx->crl_path);
-    md5_ifstr_update(hash, "crl_file:", ctx->crl_file);
-    md5_fmt_update(hash, "crl_check_mask:%d", ctx->crl_check_mask);
-    md5_fmt_update(hash, "ocsp_mask:%d", ctx->ocsp_mask);
-    md5_fmt_update(hash, "ocsp_force_default:%d", ctx->ocsp_force_default);
-    md5_ifstr_update(hash, "ocsp_responder:", ctx->ocsp_responder);
-
-#ifdef HAVE_SRP
-    md5_ifstr_update(hash, "srp_vfile:", ctx->srp_vfile);
-#endif
-
-#ifdef HAVE_SSL_CONF_CMD
-    {
-        apr_array_header_t *parms = ctx->ssl_ctx_param;
-        int n;
-
-        for (n = 0; n < parms->nelts; n++) {
-            ssl_ctx_param_t *p = &APR_ARRAY_IDX(parms, n, ssl_ctx_param_t);
-
-            md5_str_update(hash, "param:", p->name);
-            md5_str_update(hash, "value:", p->value);
-        }
-    }
-#endif
-}
-#endif
-
-static char *create_sni_policy_hash(apr_pool_t *p, apr_pool_t *ptemp,
-                                    modssl_snivhpolicy_t policy,
-                                    SSLSrvConfigRec *sc)
-{
-    char *rv = NULL;
-#ifdef HAVE_TLSEXT
-    if (policy != MODSSL_SNIVH_STRICT && policy != MODSSL_SNIVH_INSECURE) {
-        apr_md5_ctx_t hash;
-        unsigned char digest[APR_MD5_DIGESTSIZE];
-
-        /* Create the vhost policy hash for comparison later. */
-        apr_md5_init(&hash);
-        hash_sni_policy_auth(&hash, sc->server);
-        if (policy == MODSSL_SNIVH_SECURE)
-            hash_sni_policy_pk(ptemp, &hash, sc->server);
-        apr_md5_final(digest, &hash);
-
-        rv = apr_palloc(p, 2 * APR_MD5_DIGESTSIZE + 1);
-        ap_bin2hex(digest, APR_MD5_DIGESTSIZE, rv); /* sets final '\0' */
-    }
-#endif
-    return rv;
-}
-
 /*  _________________________________________________________________
 **
 **  Let other answer special connection attempts. 
@@ -319,7 +213,7 @@ int ssl_is_challenge(conn_rec *c, const char *servername,
 #ifdef HAVE_FIPS
 static apr_status_t modssl_fips_cleanup(void *data)
 {
-    FIPS_mode_set(0);
+    modssl_fips_enable(0);
     return APR_SUCCESS;
 }
 #endif
@@ -410,12 +304,6 @@ apr_status_t ssl_init_Module(apr_pool_t *p, apr_pool_t *plog,
         if (sc->server && sc->server->pphrase_dialog_type == SSL_PPTYPE_UNSET) {
             sc->server->pphrase_dialog_type = SSL_PPTYPE_BUILTIN;
         }
-
-#ifdef HAVE_FIPS
-        if (sc->fips == UNSET) {
-            sc->fips = FALSE;
-        }
-#endif
     }
 
 #if APR_HAS_THREADS && MODSSL_USE_OPENSSL_PRE_1_1_API
@@ -442,24 +330,28 @@ apr_status_t ssl_init_Module(apr_pool_t *p, apr_pool_t *plog,
     ssl_rand_seed(base_server, ptemp, SSL_RSCTX_STARTUP, "Init: ");
 
 #ifdef HAVE_FIPS
-    if (sc->fips) {
-        if (!FIPS_mode()) {
-            if (FIPS_mode_set(1)) {
-                ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, s, APLOGNO(01884)
-                             "Operating in SSL FIPS mode");
-                apr_pool_cleanup_register(p, NULL, modssl_fips_cleanup,
-                                          apr_pool_cleanup_null);
-            }
-            else {
-                ap_log_error(APLOG_MARK, APLOG_EMERG, 0, s, APLOGNO(01885) "FIPS mode failed");
-                ssl_log_ssl_error(SSLLOG_MARK, APLOG_EMERG, s);
-                return ssl_die(s);
-            }
+    if (!modssl_fips_is_enabled() && mc->fips == TRUE) {
+        if (!modssl_fips_enable(1)) {
+            ap_log_error(APLOG_MARK, APLOG_EMERG, 0, base_server, APLOGNO(01885)
+                         "Could not enable FIPS mode");
+            ssl_log_ssl_error(SSLLOG_MARK, APLOG_EMERG, base_server);
+            return ssl_die(base_server);
         }
+
+        apr_pool_cleanup_register(p, NULL, modssl_fips_cleanup,
+                                  apr_pool_cleanup_null);
+    }
+
+    /* Log actual FIPS mode which the SSL library is operating under,
+     * which may have been set outside of the mod_ssl
+     * configuration. */
+    if (modssl_fips_is_enabled()) {
+        ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, base_server, APLOGNO(01884)
+                     MODSSL_LIBRARY_NAME " has FIPS mode enabled");
     }
     else {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, APLOGNO(01886)
-                     "SSL FIPS mode disabled");
+        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, base_server, APLOGNO(01886)
+                     MODSSL_LIBRARY_NAME " has FIPS mode disabled");
     }
 #endif
 
@@ -536,8 +428,6 @@ apr_status_t ssl_init_Module(apr_pool_t *p, apr_pool_t *plog,
                 return rv;
             }
         }
-
-        sc->sni_policy_hash = create_sni_policy_hash(p, ptemp, mc->snivh_policy, sc);
     }
 
     /*
@@ -581,7 +471,7 @@ apr_status_t ssl_init_Module(apr_pool_t *p, apr_pool_t *plog,
 
 /*
  * Support for external a Crypto Device ("engine"), usually
- * a hardware accellerator card for crypto operations.
+ * a hardware accelerator card for crypto operations.
  */
 #if defined(HAVE_OPENSSL_ENGINE_H) && defined(HAVE_ENGINE_INIT)
 apr_status_t ssl_init_Engine(server_rec *s, apr_pool_t *p)
@@ -943,6 +833,14 @@ static apr_status_t ssl_init_ctx_protocol(server_rec *s,
         SSL_CTX_set_keylog_callback(ctx, modssl_callback_keylog);
     }
 #endif
+
+#ifdef SSL_OP_IGNORE_UNEXPECTED_EOF
+    /* For server-side SSL_CTX, enable ignoring unexpected EOF */
+    /* (OpenSSL 1.1.1 behavioural compatibility).. */
+    if (!mctx->pkp) {
+        SSL_CTX_set_options(ctx, SSL_OP_IGNORE_UNEXPECTED_EOF);
+    }
+#endif
     
     return APR_SUCCESS;
 }
